package lister;

import java.util.ArrayList;
import java.util.List;

import domain.CutOptions;
import domain.Motion;
import domain.ShotType;
import domain.SceneObjectType;

public class Vector {
	
	public Feature[] feature;
	public int[] option;
	int options;
	
	public List<String> maxIntAsStrings(int max) {
		List<String> strings = new ArrayList<String>();
		for (int i = 0; i < max; i ++)
			strings.add(i + "");
		return strings;
	}
	
	public Vector() {
	}
	
	public Vector(int options) {
		this.options = options;
		option = new int[options];
		feature = new Feature[Features.values().length];
		for (Features f: Features.values()) {
			switch (f) {
			case cut:
				feature[f.ordinal()] =
					new Feature(CutOptions.getStrings(), options, f.name());
				break;
			case linesSinceCut:
				feature[f.ordinal()] = 
					new Feature(maxIntAsStrings(Constants.MAXLINESSINCECUT), options, f.name());
				break;
			case sceneObjectType:
				feature[f.ordinal()] = 
					new Feature(SceneObjectType.getStrings(), options, f.name());
				break;
			case shotType:
				feature[f.ordinal()] = 
					new Feature(ShotType.getStrings(), options, f.name());
				break;
			case motions:
				feature[f.ordinal()] = 
					new Feature(Motion.getStrings(), options, f.name());
				break;
			default:
			;
			}
		}
	}
		
	public static class Feature {
//		private int total;
		public String name;
		public Selection[] selection;
		
		public Feature() {
		}
		
		public Feature(List<String> values, int options, String name) {
			int length = values.size();
			this.name = name;
			selection = new Selection[length];
			for (int i = 0; i < length; i++) {
				selection[i] = new Selection(options, values.get(i));
			}
			
		}
		public int totalSelectionGivenOption(int selPick, int optionPick) {
			return selection[selPick].option[optionPick];
		}
//		public int total() {
//			return total;
//		}
//		
//		public void increment() {
//			total++;
//		}
	}
	
	public static class Selection {
//		private int total; //total number of times this feature appears
		public String selected;
		public int[] option;
		
		public Selection() {
		}
		
		public Selection(int options, String selected) {
			this.selected = selected;
			option = new int[options];
		}
		
		public int total() {
			int total = 0;
			for (int i: option)
				total+=i;
			return total;
		}
		
	}
	
//	public class Count {
//		private int total;
//		
//		public Count() {
//			total = 0;
//		}
//		
//		public int total() {
//			return total;
//		}
//		
//		public void increment() {
//			total++;
//		}
//	}
	
	public int totalOption(int optionPick) {
		int total = 0;
		for (Feature f : feature) {
			for (Selection s: f.selection) {
				total += s.option[optionPick];
			}
		}
		return total;
	}
	
	public double probabilitySelectionGivenOption(int feat, int selPick, int optionPick) {
//		int occurences = feature[feat].totalSelectionGivenOption(selPick, optionPick);
//		return (double)occurences/(double)count[optionPick];
		double prob = 0.0;
		try {
		prob = (double)feature[feat].selection[selPick].option[optionPick]
				/(double)option[optionPick];
		} catch (Exception e) {
			System.out.println(feat + " " + selPick + " " + optionPick);
		}
		return prob;
	}
	
	public double probabilityOption(int optionPick) {
		int total = 0;
		for (int o:option)
			total += o;
		return (double)option[optionPick]/(double)total;
	}
	
	/**
	 * Returns the option with the highest probability
	 * @param sel an array where each value is the selection chosen for each feature
	 * @return the option based on Naive Bayes
	 */
	public int getOption (int[] sel) {
		int current = 0;
		double high = 0;
		for (int o = 0; o < this.option.length; o++) {
			
			double total = probabilityOption(o);
			for (int f = 0; f < this.feature.length; f++) {
				double prob = probabilitySelectionGivenOption(f, sel[f], o);
				total *= prob;
			}
			
			if (total > high) {
				current = o;
				high = total;
			}
		}
		return current;
	}
	
}
