package lister;

import java.util.ArrayList;
import java.util.List;

import javax.swing.JOptionPane;

import parser.Parser;
import util.FileUtil;

import com.fasterxml.jackson.core.JsonProcessingException;

import liner.LineData;
import liner.LinerUtil;
import lister.Vector.Feature;
import lister.Vector.Selection;
import domain.AllData;
import domain.SceneObjectType;
import domain.Script;

public class VectorUtil {
	
	public static List<Vector> createAllVectors() {
		ArrayList<Vector> vectors = new ArrayList<Vector>();
		Vector cutVector = createCutVector();
		vectors.add(cutVector);
		return vectors;
	}
	
	public static Vector createCutVector() {
		Vector v = new Vector(Constants.CUTOPTIONS);
		return v;
	}
	
	public static Vector addAllDataToCutVector(AllData data, Vector v) {
		int linesSinceCut = 0;
		int choice;
		Selection s = null;
		String sel = null;
		for (LineData line: data.getDataLines()) {
			choice = (line.isNewShot()) ? 1 : 0;
			for (Features f: Features.values()) {
				switch (f) {
					case cut:
						s = v.feature[Features.cut.ordinal()].selection[choice];
						break;
					case linesSinceCut:
						s = v.feature[Features.linesSinceCut.ordinal()].selection[linesSinceCut];
						break;
					case sceneObjectType:
						sel = data.getScript().getSceneObjects().get(data.getScript().getLines()[line.getLineNum()]).getType();
						int type = 0;
						try {
							type = SceneObjectType.fromString(sel).ordinal();
						} catch (NullPointerException e) {
							JOptionPane.showMessageDialog(null, "FromStringSOT bad line " + line.getLineNum());
						}
						//						JOptionPane.showMessageDialog(null, "Type is " + type);
						try {
							s = v.feature[Features.sceneObjectType.ordinal()].selection[type];
						} catch (NullPointerException e) {
							JOptionPane.showMessageDialog(null, "SceneObjectType bad line " + line.getLineNum());
						}
						break;
					case shotType:
						s = v.feature[Features.shotType.ordinal()].selection[line.getShotType()];
						break;
					case motions:
						try {
							s = v.feature[Features.motions.ordinal()].selection[line.getMotion()];
						} catch (Exception e) {
							JOptionPane.showMessageDialog(null, "Motion bad line " + line.getLineNum());
						}
						break;
				}
				s.option[choice] ++;
			}
			v.option[choice] ++;
			linesSinceCut = (line.isNewShot()) ? 0 : linesSinceCut + 1;
		}
		return v;	
	}
	
	public static boolean populateVectorsFile(String trainingFile) {
		String vectorFile = "vectors.json";
		String fileType = FileUtil.getFileType(trainingFile);
		
		AllData data = null;
		if (fileType.equals("zip")) {
			data = FileUtil.getAllDataFromZip(trainingFile);
		}
		else if (fileType.equals("json")) {
			data = FileUtil.getAllDataFromJson(trainingFile);
		}
		else {
			JOptionPane.showMessageDialog(null, "File must be of type .zip or .json");
			return false;
		}
		
		List<Vector> vectors = FileUtil.getVectorsFromFile(vectorFile, false);
		vectors.set(0, VectorUtil.addAllDataToCutVector(data, vectors.get(0)));
		
		String output;
		try {
			output = FileUtil.toJsonString(vectors);
		} catch (JsonProcessingException e1) {
			output = "whoops";
			throw new RuntimeException("Trouble parsing the vectors to json.");
		}
		try {
			FileUtil.writeFile(vectorFile, 
					output);
			JOptionPane.showMessageDialog(null, "Data written to " 
					+ vectorFile);
			return true;
		} catch (Exception e1) {
			throw new RuntimeException("Could not write data to file");
		}
	}
	
	public static boolean scriptFileToAllDataFile(String file) {
		String vectorFile = "vectors.json";
		String name = FileUtil.getName(file);
		List<Vector> vectors = FileUtil.getVectorsFromFile(vectorFile, true);
		if (vectors == null) {
			return false;
		}
		
		AllData allData;
		ArrayList<String> text = FileUtil.readFile(file);
		Script script = Parser.textToScript(text, name);
		allData = LinerUtil.createDataLines(text, script);
		if(LinerUtil.recheckScriptObjects(allData))
			LinerUtil.updateDataLines(allData);
		
		allData = naiveBayes(allData, vectors);
		
		try {
			FileUtil.writeZipFile(name, FileUtil.toJsonString(allData));
			JOptionPane.showMessageDialog(null, "Shot list created and written to " 
					+ name + ".zip");
			return true;
		} catch (Exception e) {
			JOptionPane.showMessageDialog(null, "Couldn't write shots to file");
			return false;
		}
	}
	
	public static AllData naiveBayes(AllData data, List<Vector> vectors) {
		Vector cutVector = vectors.get(0);
		int linesSinceCut = 0;
		
		for (LineData line:data.getDataLines()) {
			int[] selections = new int[Features.values().length];
			int feature = 0;
			for (Features f: Features.values()) {
				switch (f) {
				case cut:
					selections[feature] = (line.isNewShot()) ? 1 : 0;
					break;
				case linesSinceCut:
					selections[feature] = linesSinceCut;
					break;
				case sceneObjectType:
					selections[feature] = SceneObjectType.fromString(
							data.getScript().getSceneObjects().get(
									data.getScript().getLines()[line.getLineNum()])
									.getType()).ordinal();
					break;
				case shotType:
					selections[feature] = line.getShotType();
					break;
				case motions:
					selections[feature] = line.getMotion();
					break;
				}
				feature++;
			}
			for (int sel: selections) {
				System.out.print(sel + " ");
			}
			System.out.println();
			int option = cutVector.getOption(selections);
			line.setNewShot(selections[0] == 1 || option == 1);
			linesSinceCut = (line.isNewShot()) ? 0 : linesSinceCut + 1;
			if (linesSinceCut >= Constants.MAXLINESSINCECUT)
				linesSinceCut = Constants.MAXLINESSINCECUT - 1;
		}
		return data;
	}
}
