package util;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import javax.swing.JOptionPane;

import liner.LinerUtil;
import lister.Vector;
import lister.VectorUtil;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import domain.AllData;

public class FileUtil {

	static ObjectMapper mapper = new ObjectMapper();

	public static String toJsonString(Object o) throws JsonProcessingException {
		return mapper.writerWithDefaultPrettyPrinter().writeValueAsString(o);
	}

	public static AllData getAllData(InputStream is) {
		AllData data;
		try {
			data = mapper.readValue(is, AllData.class);
		} catch (Exception e) {
			JOptionPane.showMessageDialog(null, 
					"Couldn't map json file. Make sure you're using the correct json file and that it hasn't been corrupted.");
			e.printStackTrace();
			throw new RuntimeException("Couldn't map json file. Make sure you're using the correct json file and that it hasn't been corrupted.");
			
		}
		LinerUtil.restoreAllData(data);
		return data;
	}

	public static AllData getAllDataFromJson(String file) {
		InputStream is = getInputStream(file);
		AllData data = getAllData(is);
		return data;
	}
	
	public static AllData getAllDataFromZip(String file) {
		InputStream is = getInputStreamFromZip(file);
		return getAllData(is);
	}
	
	public static InputStream getInputStreamFromZip(String file) {
		ZipInputStream zis = null;
		try {
			FileInputStream fis = new FileInputStream(file);
			zis = new ZipInputStream(new BufferedInputStream(fis));
			zis.getNextEntry();
		} catch (Exception e1) {
			throw new RuntimeException("Couldn't read zip file");
		}
		return zis;
	}
	
	public static InputStream getInputStream(String file) {
		return getInputStream(new File(file));
	}
	
	public static InputStream getInputStream(File file) {
		InputStream is;
		try {
			is = new FileInputStream(file);
		} catch (FileNotFoundException e) {
			throw new RuntimeException("Couldn't read file.");
		}
		return is;
	}

	public static ArrayList<String> readFile(String file) {
		ArrayList<String> lines = new ArrayList<>();
		FileInputStream fstream = null;
		BufferedReader br = null;
		int counter = 0;
		try {
			fstream = new FileInputStream(file);
		} catch (FileNotFoundException e) {
			System.out.println("The file " + file + " was not found.");
			return null;
		}
		br = new BufferedReader(new InputStreamReader(fstream));
	
		String line;
		try {
			while ((line = br.readLine()) != null)   {
				counter++;
				lines.add(line);
			}
			br.close();
		} catch (IOException e) {
			System.out.println("There was an IO error reading the file on line " + counter);
			return null;
		}
	
		return lines;
	}
	
	public static List<Vector> getVectorsFromFile(String file, boolean strict) {
		
		File f = new File(file);
		if (f.exists()) {
			InputStream is = getInputStream(f);
			ArrayList<Vector> vectors;
			try {
				vectors = mapper.readValue(is, new TypeReference<List<Vector>>(){});
			} catch (Exception e) {
				JOptionPane.showMessageDialog(null, 
						"Couldn't map json file. Make sure you're using the correct json file and that it hasn't been corrupted.");
				e.printStackTrace();
				throw new RuntimeException("Couldn't map json file. Make sure you're using the correct json file and that it hasn't been corrupted.");
			}
			return vectors;
		}
		else {
			if (strict) {
				JOptionPane.showMessageDialog(null, "No Vectors file found");
				return null;
			}
			else {
				JOptionPane.showMessageDialog(null, "Creating new Vectors File");
				return VectorUtil.createAllVectors();
			}
		}
	}

//	public static String compress(String str) throws IOException {
//        if (str == null || str.isEmpty()) {
//            return str;
//        }
//        ByteArrayOutputStream out = new ByteArrayOutputStream();
//        GZIPOutputStream gzip = new GZIPOutputStream(out);
//        gzip.write(str.getBytes());
//        gzip.close();
//        String outStr = out.toString("ISO-8859-1");
//        return outStr;
//    }
	
	public static void writeFile(String filename, String data) throws FileNotFoundException {
		PrintWriter out = new PrintWriter(filename);
		out.println(data);
		out.close();
	}
	
	public static void writeZipFile(String baseFilename, String data) throws Exception {
		final File f = new File(baseFilename + ".zip");
		final ZipOutputStream zOut = new ZipOutputStream(new FileOutputStream(f));
		ZipEntry entry = new ZipEntry(baseFilename + ".json");
		zOut.putNextEntry(entry);

		byte[] outData = data.getBytes();
		zOut.write(outData, 0, outData.length);
		zOut.closeEntry();
		zOut.close();
	}

	public static String getName(String filePath) {
		String name = filePath;
		if (name.contains("/")) {
			String fileSplit[] = name.split("/");
			name = fileSplit[fileSplit.length-1];
		}
		else if (name.contains("\\")) {
			System.out.println(name);
			String fileSplit[] = name.split("\\\\");
			name = fileSplit[fileSplit.length-1];
		}
		if (name.contains(".")) {
			name = name.split("\\.")[0];
		}
		return name;
	}

	public static String getFileType(String filePath) {
		String name = filePath;
		if (!name.contains("."))
			return null;
		String[] split = name.split("\\.");
		return split[split.length-1].toLowerCase();
	}

}
