package util;
import java.util.ArrayList;
import java.util.Random;

import domain.ColorValue;
import domain.Scene;


public class Util {
	
	public static Random random = new Random();
	
	public static String trimLead(String s) {
	    return s.replaceAll("^\\s+", "");
	}
	
	public static boolean isFirstAndLastNum(String line) {
		String[] words = line.replaceAll("^\\s+", "").trim().split(" ");
		return (words[0].equals(words[words.length-1])) && isNumeric(words[0]);
	}
	
	public static String removeFirstAndLastNum(String line) {
		String[] words = line.replaceAll("^\\s+", "").trim().split(" ");
		if (!words[0].startsWith("INT") 
				&& !words[0].startsWith("EXT")
				&& isNumeric(words[0].substring(0,1))) {
			line = line.replaceAll(words[0], " ");
		}
//		if (words[0].equals(words[words.length-1]) && isNumeric(words[0])) {
//			StringBuilder str = new StringBuilder();
//			for (int i = 1; i < words.length -1; i++) {
//				str.append(words[i]);
//				if (i < words.length -2)
//					str.append(" ");
//			}
//			return str.toString();
//		}
		return line;
	}
	
	public static boolean isScene(String inLine) {
		String line = inLine.replaceAll("^\\s+", "");
		boolean b = (line.startsWith("INT ") || 
				line.startsWith("INT. ") ||
				line.startsWith("INTERIOR") ||
				line.startsWith("EXT ") ||
				line.startsWith("EXT.") ||
				line.startsWith("EXTERIOR") ||
				line.indexOf("INT.") != -1 ||
				line.indexOf("EXT.") != -1) && isMostlyUpper(line, .9);
		return b || isFirstAndLastNum(line);
	}
	
	public static boolean isBlank(String line) {
		return line == null || line.trim().isEmpty();
	}
	
	public static boolean isAllUpper(String line) {
		boolean hasLower = false;
		boolean hasUpper = false;
		for (char c: line.toCharArray()) {
			if (Character.isLowerCase(c))
				hasLower=true;
			if (Character.isUpperCase(c))
				hasUpper=true;
		}
		return hasUpper && !hasLower;
	}
	
	/**
	 * Tests if a line is mostly uppercase letters based on a threshold percentage
	 * @param line
	 * @param threshold
	 * @return
	 */
	public static boolean isMostlyUpper(String line, double threshold) {
		double uppers = 0;
		double lowers = 0;
		for (char c: line.toCharArray()) {
			if (Character.isLowerCase(c))
				lowers++;
			if (Character.isUpperCase(c))
				uppers++;
		}
		if (uppers+lowers == 0)
			return false;
		double percentage = uppers/(uppers+lowers);
		return (percentage >= threshold);
	}
	
	public static ArrayList<String> getUpperWords(String line) {
		ArrayList<String> upperWords = new ArrayList<String>();
		String[] allWords = line.split(" ");
		boolean lastUpper = false;
		for (int i=0;i<allWords.length;i++) {
			if(isAllUpper(allWords[i])
					&& !allWords[i].equals("I")
					&& !allWords[i].equals("A")) {
				if (lastUpper) 
					upperWords.set(upperWords.size()-1, upperWords.get(upperWords.size()-1) + " " + allWords[i].trim());
				else
					upperWords.add(allWords[i].trim());
				lastUpper = true;
			}
			else
				lastUpper = false;
		}
		return upperWords;
	}
	
	public static boolean isCharacter(ArrayList<String> text, int index, int lastLine) {

		return index>0 && index<lastLine && isAllUpper(text.get(index)) &&  isBlank(text.get(index-1)) 
				&& !isBlank(text.get(index+1)) && !isScene(text.get(index+1));
	}
	
	public static boolean isTransition(String line, String prev, String next) {
		return isAllUpper(line) && isBlank(prev) && isBlank(next) && line.trim().endsWith(":");
	}
	
	/**
	 * Removes any strange characters from the end of a string, including periods and commas
	 * Removes all paranthesis
	 * @param in
	 * @return
	 */
	public static String clean(String in) {
		String out = in;
		if (in.endsWith(".") 
				|| in.endsWith(",")
				|| in.endsWith("!")
				|| in.endsWith("?"))
			out = in.substring(0, in.length()-1);
		out = out.replaceAll("[()]", "");
		return out;
	}
	
	public static boolean isWhen(String line) {
		line = line.trim();
		return (line.equals("DAY") ||
				line.equals("NIGHT") ||
				line.equals("DAWN") ||
				line.equals("MORNING") ||
				line.equals("EVENING") ||
				line.equals("DUSK") || 
				line.equals("LATER"));
	}
	
	public static Scene parseSceneString(String line) {
		Scene scene = new Scene();
		line = removeFirstAndLastNum(line);
		String[] whereSplit = line.trim().split(" ", 2);
		while (!whereSplit[0].startsWith("INT") 
				&& !whereSplit[0].startsWith("EXT") 
				&& whereSplit.length == 2) {
			whereSplit = whereSplit[1].split(" ", 2);
		}
		if (whereSplit.length < 2) {
			scene.setWhere("");
			scene.setLocation(line);
			scene.setWhen("");
		}
		else {
			scene.setWhere(whereSplit[0]);
			String next = whereSplit[1];
			next = next.replaceAll("--", "-");
			if (next.contains("-")) {
				String[] whenSplit = next.split("\\-");
				String when = whenSplit[whenSplit.length-1].trim();
				if (isWhen(when))
					scene.setWhen(when);
				else
					scene.setWhen("");
				String location = whenSplit[0].trim();
				for (int i=1; i<whenSplit.length; i++)
					if(!isWhen(whenSplit[i]))
						location = location + " - " + whenSplit[i].trim();
				scene.setLocation(location);
			}
			else {
				scene.setLocation(next.trim());
				scene.setWhen("");
			}
		}
		return scene;
	}

	public static String increaseShot(String ctr) {
		if (ctr == null)
			return "A";
		int len = ctr.length();
		char[] num = new char[len];
		for (int i=0; i< len; i++)
			num[i] = ctr.charAt(i);
		StringBuilder newShot = new StringBuilder();
		if (num[len-1] < 'Z') {
			num[len-1]++;
		}
		else {
			int pos = len -1;
			while (num[pos] == 'Z') {
				num[pos] = 'A';
				pos--;
				if (pos < 0) {
					newShot.append("A");
					break;
				}
			}
			if (pos>=0 && num[pos] < 'Z')
				num[pos]++;
		}
		for (int i=0; i<len; i++)
			newShot.append(num[i]);
		return newShot.toString();
	}

	public static boolean isNumeric(String str)
	{
		return str.matches("-?\\d+(\\.\\d+)?");
	}
	
	public static ColorValue randomColor() {
		ColorValue color = new ColorValue(random.nextFloat(), random.nextFloat(), random.nextFloat());
		return color;
	}
	
	public static String strippedDown(String s) {
		return s.toLowerCase()
				.replace(",", "")
				.replace(".", "")
				.replace("!", "")
				.replaceAll("-", "")
				.replace("?", "")
				.replace("the ", "")
				.replace("(v.o.)", "")
				.replace("\\(", "")
				.replace("\\)", "")
				.trim();
				
	}
}
